#!/usr/bin/env python

"""ticalc.org package manager

Usage: python calcpkg.py [command] [text]

Supported commands:
	count [number of search results | takes raw text arg]
	get  [downloads all files from a ticalc.org search | takes raw text arg]
	help [prints help text | takes no args]
	list [searches ticalc.org | takes raw text enclosed arg]
	update [updates cached package lists | takes no args]
	
calcpkg is a 'package manager' for ticalc.org. This means that it can download
and search for files on ticalc.org via the command line. This is currently a
beta release.

It doesn't actually "search" ticalc.org at all, however. I've provided two
cached package lists (produced from parsing the master index on ticalc.org)
that are up to date as of 11/28/10. The program will run fine off of these.

But, if you want to update the cached package lists, you can run the update
command, which will do just that. (All it does is write the names of files
to names.index and the file paths to files.index).

The get, list, and number commands use these indexes to search and download
programs. The list command searches the files for the [text] argument. The 
[text] argument must be raw text, enclosed by ""s (the quotation marks are
not necessary, but allow you to have spaces in your search text. The output
of list is formatted as such:

File Path		Filename
========================

The get command begins by calling list. Then, it prompts you to download all
the listed files. You may wish to narrow your search down before actually
performing get (or, at least, agreeing to download the programs).

number also searches the indexes, but simply tells you how many files it found
and exits.

Future commands may include user profile access of some sort, and possibly
a command that actually does use the websearch (just in case you wish to
compare results)"""

import getopt
import os
import sys
import urllib

#Globals
fileindex = "files.index"
nameindex = "names.index"

def structureSearchOutput(fileurl, filename):
	"""Formats the output of a calcpkg list command"""
	output = fileurl
	if len(output) > 60:
		output += " " + fileurl
	else:
		while len(output) <= 60:
			output += " "
		output += filename
	return output

def ticalcUpdatePackageLists(prompt=True):
	"""Refreshes local copies of ticalc index files"""
	global fileindex
	global nameindex
	
	#Confirm the user really wants to do it
	if prompt:
		confirm = raw_input("Warning! This process may take some time to run. Are you sure you wish to sync package lists right now [Y/N]? ")
		if confirm.lower() != 'y':
			print "Operation aborted by user input"
			return
		
	#Now... do it!
	print ""
	print "Reading ticalc.org master index (this will take some time)"
	
	#First read in the text (the only network process involved)
	masterindex = urllib.urlopen('http://www.ticalc.org/pub/master.index').read()
	print "Read in ticalc.org master index"
	print ""
	
	#Now, try to delete the indexes on system
	try:
		os.remove(fileindex)
		print "Deleted old file index"
	except:
		print "No files index found"
	try:
		os.remove(nameindex)
		print "Deleted old name index"
	except:
		print "No names index found"
		
	#Now, try to open new indexes to write to
	try:
		files = open(fileindex, 'wt')
	except:
		print "ERROR: Unable to create file " + nameindex + " in current folder. Quitting."
		return
	try:
		names = open(nameindex, 'wt')
	except:
		print "ERROR: Unable to create file " + nameindex + " in current folder. Quitting."
		files.close()
		return
	
	#Now, parse the enormous data and write index files
	print ""
	masterindex = masterindex[39:]
	dir = ""
	while len(masterindex) > 2:
		line = masterindex[:masterindex.find('\n')]
		masterindex = masterindex[masterindex.find('\n') + 1:]
		if line == "":
			continue
		if line[:9] == "Index of ":
			dirdata = line[9:]
			dir = dirdata[:dirdata.find(" ")]
			print "Now caching packages in " + line[9:]
		else:
			filedata = line[:line.find(" ")]
			files.write(dir + '/' + filedata + '\n')
			namedata = line[len(filedata)+1:].lstrip()
			names.write(namedata + '\n')
	
	#Close the indexes now
	files.close()
	names.close()
	
def ticalcSearchIndex(string):
	"""Core function to search the indexes of ticalc.org and return data for get"""
	global nameindex
	global fileindex
	data = []
	namedata = {}
	filedata = {}
	
	#Search the name index
	try:
		namefile = open(nameindex, 'rt')
	except IOError:
		print "ERROR: You should try running calcpkg.py update first - there was an error reading the name index"""
		return
	count = 1
	for line in namefile:
		count += 1
		if string in line:
			namedata[count] = line[:len(line) - 1]
	namefile.close()
	
	#Now use the name index search to search the file index
	try:
		filefile = open(fileindex, 'rt')
	except IOError:
		print "ERROR: You should try running calcpkg.py update first - there was an error reading the file index"""
		return
	count = 1
	for line in filefile:
		count += 1
		try:
			if namedata[count] != None:
				filedata[count] = line[:len(line) - 1]
		except:
			pass
	filefile.close()	
	
	#Prepare output to parse
	for key, value in namedata.iteritems():
		value2 = filedata[key]
		data.append([value2, value])
	
	#Print output
	print ""
	print structureSearchOutput("File Path/Category:", "File Name:")
	print "======================================================================================================================="
	for datum in data:
		print structureSearchOutput(datum[0], datum[1])
		
	#Return data
	return data
	
def ticalcDownload(string):
	"""Runs a list and then downloads all the listed files to your computer."""

	data = ticalcSearchIndex(string)
	print ""
	
	#First, confirm the user wants to do this
	confirm = raw_input("Are you sure you want to download all these file(s) [Y/N]? ")
	if confirm.lower() != 'y':
		print "Operation aborted by user input"
		return
	
	#Now, download the files
	for datum in data:
		download = 'http://www.ticalc.org' + datum[0]
		print ""
		print "Downloading " + datum[0] + " from " + download
		filedata = urllib.urlopen(download).read()
		downame = datum[0]
		a, b, downame = downame.rpartition('/')
		try:
			downloaded = open(downame, 'wb')
		except:
			os.remove(downame)
		downloaded.write(filedata)
		downloaded.close()
		print "Download complete!"

def ticalcCount(string):
	"""Counts the number of ticalc.org files containing some search term, doesn't return them"""
	global nameindex
	try:
		findex = open(nameindex)
	except:
		print "ERROR: No name index found. Please run calcpkg.py update"
	index = findex.read()
	count = index.lower().count(string.lower())
	findex.close()
	print ""
	print "Search for '" + string + "' returned " + str(count) + ' result(s)'
	return "Search for '" + string + "' returned " + str(count) + ' result(s)'
		
def main():
	"""Core function for the script"""
	#List of commands
	commands = ['get', 'list', 'help', 'count', 'update']
	
	#Try to get a command
	try:
		command = sys.argv[1]
	except:
		print >> sys.stderr, __doc__
		return
	if not command in commands:
		print "Unrecognized command, exiting"
		return

	#Command without an argument
	if command == 'help':
		print >> sys.stderr, __doc__
		return
	if command == 'update':
		ticalcUpdatePackageLists()
		return
	
	#Second argument
	try:
		stringarg = sys.argv[2]
	except:
		print "Error retrieving get/list/number argument"
		return
		
	#Commands with an argument
	if command == 'get':
		ticalcDownload(stringarg)
	if command == 'list':
		ticalcSearchIndex(stringarg)
	if command == 'count':
		ticalcCount(stringarg)
	
if __name__ == "__main__":
	main()
